//! Color constants to replace more verbose methods of defining colors in the game.
//! These constants replace the `::rltk::RGB` constructor functions
use ::rltk::RGB;

const fn new(r: f32, g: f32, b: f32) -> RGB {
    RGB { r, g, b }
}

//Grays

/// HEX: #000
/// RGB: (0, 0, 0)
pub const BLACK: RGB = new(0., 0., 0.);

/// RGB: (102, 102, 102)
pub const DARK_GRAY: RGB = new(0.4, 0.4, 0.4);

/// RGB: (128, 128, 128)
pub const MID_GRAY: RGB = new(0.5, 0.5, 0.5);

/// HEX: #999
/// RGB: (153, 153, 153)
pub const BOX_GRAY: RGB = new(0.6, 0.6, 0.6);

/// RGB: (190, 190, 190)
pub const GRAY: RGB = new(0.745, 0.745, 0.745);

/// HEX: #CCC
/// RGB: (204, 204, 204)
pub const ATTR_GRAY: RGB = new(0.8, 0.8, 0.8);

/// Hex: #DDD
/// RGB: (221, 221, 221)
pub const LIGHT_GRAY: RGB = new(0.86667, 0.86667, 0.86667);

/// Hex: #FFF
/// RGB: (255, 255, 255)
pub const WHITE: RGB = new(1., 1., 1.);

// Non-grays

/// RGB: (191, 0, 0)
pub const BLOOD: RGB = new(0.75, 0., 0.);

/// HEX: #00F
/// RGB: (0, 0, 255)
pub const BLUE: RGB = new(0., 0., 1.);

/// RGB: (210, 105, 30)
pub const CHOCOLATE: RGB = new(0.83253, 0.41176, 0.11765);

/// HEX: #0FF
/// RGB: (0, 255, 255)
pub const CYAN: RGB = new(0., 1., 1.);

/// RGB: (0, 128, 128)
pub const DEFAULT_FLOOR: RGB = new(0., 0.5, 0.5);

/// RGB: (51, 51, 255)
pub const DEEP_WATER: RGB = new(0.2, 0.2, 1.0);

/// RGB: (0, 153, 0)
pub const FOREST_GREEN: RGB = new(0., 0.6, 0.);

/// RGB: (255, 215, 0)
pub const GOLD: RGB = new(1., 0.84314, 0.);

/// HEX: #0F0
/// RGB: (0, 255, 0)
pub const GREEN: RGB = new(0., 1., 0.);

/// Hex: #F0F
/// RGB: (255, 0, 255)
pub const MAGENTA: RGB = new(1., 0., 1.);

/// RGB: (255, 165, 0)
pub const ORANGE: RGB = new(1., 0.647, 0.);

/// Hex: #F00
/// RGB: (255, 0, 0)
pub const RED: RGB = new(1., 0., 0.);

/// RGB: (255, 255, 128)
pub const TORCH_LIGHT: RGB = new(1., 1., 0.5);

/// RGB: (245, 222, 179)
pub const WHEAT: RGB = new(0.96078, 0.87059, 0.70196);

/// Hex: #FF0
/// RGB: (255, 255, 0)
pub const YELLOW: RGB = new(1., 1., 0.);

// Equipment colors
pub const EQUIP_COMMON: RGB = new(0.5, 1., 0.5);
pub const EQUIP_RARE: RGB = new(0., 1., 1.);
pub const EQUIP_LEGEND: RGB = new(0.71, 0.15, 0.93);
